' OpenB3D Framework
' (C) 2017 by Krischan
' Feel free to contact me at  www.blitzbasic.com
' Get the OpenB3D Wrapper at  https://github.com/markcwm/openb3d.mod
' Get Bruceys bha.DBSQLite at http://brucey.net/programming/blitz/

SuperStrict

Import "include/icon/blitz.o"

Framework Openb3d.B3dglgraphics										' OpenB3D Framework

Import brl.TGAloader												' TGA extension
Import brl.JPGloader												' JPG extension
Import brl.OGGloader												' OGG extension

Import brl.Pixmap													' Pixmap extension
Import brl.Random													' Random extension
Import brl.timer													' Timer extension
Import brl.EventQueue												' Events extension
Import brl.DirectSoundAudio											' Sounddriver extension
Import brl.FreeTypeFont												' Freetype Font extension
Import brl.Map														' TMap extension

Import bah.DBSQLite													' SQlite database extension

Import maxgui.Drivers												' MAXGUI extension
Import maxgui.ProxyGadgets											' MAXGUI extension

EnablePolledInput()



' ------------------------------------------------------------------------------------------------
' Audio Driver
' ------------------------------------------------------------------------------------------------
?Win32
SetAudioDriver "directsound"
?MacOs
SetAudioDriver "freeaudio"
?



' ------------------------------------------------------------------------------------------------
' Sourcecode Includes
' ------------------------------------------------------------------------------------------------

' System
Include "include/GUI.bmx"											' GUI functions
Include "include/Init.bmx"											' Initialization functions
Include "include/Media.bmx"											' Media loading functions
Include "include/System.bmx"										' System functions
Include "include/Variables.bmx"										' Variable declarations

' Functions
Include "include/Functions/F3D.bmx"									' 3D Texture functions
Include "include/Functions/FBanks.bmx"								' Bank functions
Include "include/Functions/FMouse.bmx"								' Mouse functions
Include "include/Functions/FLog.bmx"								' Math functions
Include "include/Functions/FMath.bmx"								' Log functions
Include "include/Functions/FMAXGUI.bmx"								' MAXGUI functions
Include "include/Functions/FNoise.bmx"								' Noise functions
Include "include/Functions/FPixel.bmx"								' Pixel functions
Include "include/Functions/FPixmap.bmx"								' Pixmap functions
Include "include/Functions/FQuaternion.bmx"							' Quaternion functions
Include "include/Functions/FText.bmx"								' Text Formatting functions
Include "include/Functions/FRandom.bmx"								' Random functions
Include "include/Functions/FWin32.bmx"								' Windows functions

' Types
Include "include/Types/TBeat.bmx"									' TBeat Timer Type
Include "include/Types/TCam.bmx"									' Camera Type
Include "include/Types/TConfig.bmx"									' Config Type
Include "include/Types/TFlag.bmx"									' Flag Type
Include "include/Types/TFPS.bmx"									' FPS Type
Include "include/Types/TGradient.bmx"								' Gradient Type
Include "include/Types/TInfotext.bmx"								' Infotext Type
Include "include/Types/TLocale.bmx"									' Locale Type
Include "include/Types/TMedia.bmx"									' Media Management Type
Include "include/Types/TQuad.bmx"									' Single Surface Quads Type
Include "include/Types/TSimplex.bmx"								' Simplex Noise Type
Include "include/Types/TTime.bmx"									' Timing Type


' ------------------------------------------------------------------------------------------------
' Initialize Game
' ------------------------------------------------------------------------------------------------
InitGame()



' ------------------------------------------------------------------------------------------------
' Main Loop
' ------------------------------------------------------------------------------------------------
Global initializing:Int = MilliSecs()

AddHook EmitEventHook, EventHook

While Not AppTerminate()

Wend

End



' ------------------------------------------------------------------------------------------------
' Update Game Logic
' ------------------------------------------------------------------------------------------------
Function MainLoop()

	' read current time
	TIMER_Timer = MilliSecs()

	' check if update is allowed
	If (TIMER_Timer > TIMER_Update + TIMER_Lasttimer) Then

		' calculate new timer interval		
		TIMER_Interval = TIMER_Timer - TIMER_Lasttimer

		If TIMER_Interval > TIMER_Safe Then

			' check for framedrops
			TIMER_Interval = TIMER_Safe
			FPS.framedrops:+1

		End If

		FPS.multi = GAME_SPEED * TIMER_Interval

		' Update Camera
		CAMERA.Move(FLAG.Get("CAMMODE"))
		'CAMERA.zoom = MouseWheel

		CAMERA.Update()
		CAMERA.GetCameraPosition

		' store last timer
		TIMER_Lasttimer = TIMER_Timer

	EndIf

End Function



' ------------------------------------------------------------------------------------------------
' Events
' ------------------------------------------------------------------------------------------------
Function EventHook:Object(id:Int, Data:Object, Context:Object)

	Local Event:TEvent = TEvent(data)
	If Event = Null Then Return Event

	Select Event.id

		Case EVENT_APPSUSPEND
									FLAG.Set("SUSPEND", 1)
									LogUpdate(LOCALE.Get("DEBUG_APPSUSPENDED"))

		Case EVENT_APPRESUME
									FLAG.Set("SUSPEND", 0)
									LogUpdate(LOCALE.Get("DEBUG_APPRESUMED"))

		Case EVENT_WINDOWCLOSE

									End

		Case EVENT_APPTERMINATE

									End

		Case EVENT_TIMERTICK

									Select Event.Source

										Case SCREEN_TIMER

											RedrawGadget MAXGUI_CANVAS

									End Select

		Case EVENT_GADGETPAINT

									SetGraphics CanvasGraphics(MAXGUI_CANVAS)

									If FLAG.Get("SUSPEND") = 0 Then

										MainLoop()

										TESTBUMP = FLAG.Get("BUMPMAP")
										
										TESTCOUNTER:+(10.0 * FPS.multi)
										PositionEntity TESTSPHERE, 0, 1.25 + Sin(TESTCOUNTER), 10
										TurnEntity TESTSPHERE, 1 * FPS.multi, 2 * FPS.multi, 3 * FPS.Multi

										' wireframe mode
										Wireframe FLAG.Get("WIREFRAME")

										PointEntity TESTLIGHT, TESTTARGET
										LightRange AMBLIGHT, 0
										LightRange TESTLIGHT, 0									
										If FLAG.Get("LIGHTSWITCH1") Then LightRange AMBLIGHT, 1000 * MouseWheel
										If FLAG.Get("LIGHTSWITCH2") Then LightRange TESTLIGHT, 2000 * MouseWheel

										UpdateWorld
										RenderWorld

										' calculate FPS
										FPS.Update()

										' 2D GUI output
										GUI()

										' Memory Garbage Collection
										GCCollect()

										' Bufferflip
										Flip FLAG.Get("VSYNC")

									EndIf

									' let the OS breathe
									Delay FLAG.Get("DELAY")

		Case EVENT_KEYDOWN
									'LogUpdate("Key " + Event.Data + " pressed")

									Select Event.Data

										' Resolution
										Case KEY_1
											SetResolution(800, 600)

										Case KEY_2
											SetResolution(1024, 768)

										Case KEY_3
											SetResolution(1280, 720)

										Case KEY_4
											SetResolution(1920, 1080)

										Case KEY_5
											SetResolution(SCREEN_DESKTOPWIDTH, SCREEN_DESKTOPHEIGHT, True)

										Case KEY_6
											SetResolution(SCREEN_DESKTOPWIDTH, SCREEN_DESKTOPHEIGHT - GetTaskBarHeight(), False)

										' System
										Case KEY_ESCAPE
											End

										Case KEY_SPACE
											FLAG.Switch("WIREFRAME")

										Case KEY_ENTER
											InitBlip()

										Case KEY_I
											FLAG.Switch("GUI")

										Case KEY_C
											RotateEntity CAMERA.parent, 0, 0, 0
											RotateEntity CAMERA.cam, 0, 0, 0
											PositionEntity CAMERA.parent, 0, 0, 0
											PositionEntity CAMERA.cam, 0, 0, 0
											CAMERA.X = 0
											CAMERA.Y = 0
											CAMERA.Z = 0
											CAMERA.Xspeed = 0
											CAMERA.Yspeed = 0
											CAMERA.Zspeed = 0
											CAMERA.OPitch = 0
											CAMERA.OYaw = 0
											CAMERA.ORoll = 0
											CAMERA.pitch = 0
											CAMERA.Yaw = 0
											CAMERA.Roll = 0
											CAMERA.Update
											centermouse()
											FLAG.Switch("CAMMODE")

										Case KEY_B
											TESTBUMP = FLAG.Switch("BUMPMAP")

										Case KEY_NUMSUBTRACT
											Screenshot()

										Case KEY_F1
											LOCALE.language = "EN"
											LOCALE.Update
											InitLocaleArrays

										Case KEY_F2
											LOCALE.language = "DE"
											LOCALE.Update
											InitLocaleArrays

										Case KEY_TAB
											' switch Flag
											FLAG.Switch("QUALITY")

											' set variable
											If FLAG.Get("QUALITY") = 0 Then

												GAME_QUALITY = "LQ"

											Else

												GAME_QUALITY = "HQ"

											End If

											' create path according to qualitysetting
											PATH_MODEL = "models/" + GAME_QUALITY + "/"
											PATH_TEXTURE = "textures/" + GAME_QUALITY + "/"
											PATH_SOUND = "sounds/" + GAME_QUALITY + "/"

											' release old textures
											MEDIA.Unload("TEST")
											MEDIA.Unload("TEST_NRM")

											' load new textures
											TIME.Start()
											MEDIA.Load("TEX", "TEST", "test.png", 1)
											MEDIA.Load("TEX", "TEST_NRM", "test_NRM.png", 1)

											' update shader
											Local shader:TShader = MEDIA.GetShader("TESTSHADER")
											ShaderTexture(shader, MEDIA.GetTexture("TEST"), "colorMap", 0)
											ShaderTexture(shader, MEDIA.GetTexture("TEST_NRM"), "normalMap", 1)
											ShadeEntity(TESTLEVEL, shader)
											ShadeEntity(TESTSPHERE, shader)
											SetShaderProperties(shader)

										' Flags
										Case KEY_V
											FLAG.Switch("VSYNC")
											FPS.averagefps = 0

										Case KEY_F11
											FLAG.Switch("CURSOR")

										Case KEY_F12
											FLAG.Switch("FPS")

										Case KEY_CFLEX
											FLAG.Switch("DELAY")

										' Movement
										Case KEY_W
											PlayerZSpeed = 1
										Case KEY_S
											PlayerZSpeed = -1
										Case KEY_A
											PlayerXSpeed = -1
										Case KEY_D
											PlayerXSpeed = 1
										Case KEY_R
											PlayerYSpeed = 1
										Case KEY_F
											PlayerYSpeed = -1
										Case KEY_X
											PlayerZSpeed = 0
											PlayerYSpeed = 0
											PlayerXSpeed = 0

									End Select

		Case EVENT_KEYUP
									'LogUpdate("Key " + Event.Data + " released")

									Select Event.Data

										' Movement
										Case KEY_W
											PlayerZSpeed = 0
										Case KEY_S
											PlayerZSpeed = 0
										Case KEY_A
											PlayerXSpeed = 0
										Case KEY_D
											PlayerXSpeed = 0
										Case KEY_R
											PlayerYSpeed = 0
										Case KEY_F
											PlayerYSpeed = 0

									End Select

		Case EVENT_MOUSEDOWN
									MouseButton = Event.Data
									'LogUpdate("Mousebutton " + MouseButton + " pressed")

		Case EVENT_MOUSEUP
									MouseButton = Event.Data
									'LogUpdate("Mousebutton " + MouseButton + " released")

									Select Event.Data

										Case 1
											FLAG.Switch("LIGHTSWITCH1")

										Case 2
											FLAG.Switch("LIGHTSWITCH2")

									End Select

		Case EVENT_MOUSEWHEEL

									If Event.Data > 0 Then

										MouseWheel:+1
										If MouseWheel >= 5 Then MouseWheel = 5
										'LogUpdate("Mousewheel up: " + MouseWheel)

										Else

										MouseWheel:-1
										If MouseWheel < 1 Then MouseWheel = 1
										'LogUpdate("Mousewheel down: " + MouseWheel)

									EndIf


		Case EVENT_MOUSEMOVE
									CAMERA.MX = Event.x
									CAMERA.MY = Event.y
									'LogUpdate("Mouse moving To " + CAMERA.MX + "x" + CAMERA.MY)

	End Select

End Function