' ------------------------------------------------------------------------------------------------
' 2D GUI Output
' ------------------------------------------------------------------------------------------------
Function GUI()

	Begin2D()

		If FLAG.Get("GUI") Then

			' --------------------------------------------------------------------------------------------
			' System Information
			' --------------------------------------------------------------------------------------------
			Font("SMALL")

			' formatted output of informations
			info.Scale = (MEDIA.GetOption("SMALL") * Scaler)
			info.RGB = [0, 255, 0]
			info.shadow = True
			info.shadowoffset = 3 * Scaler
			If info.shadowoffset < 1 Then info.shadowoffset = 1

			info.Add(FillSpaces(LOCALE.Get("INFO_RESOLUTION")) + ": " + SCREEN_WIDTH + "x" + SCREEN_HEIGHT)
			info.Add(FillSpaces(LOCALE.Get("INFO_MOUSECOORDINATES")) + ": " + CAMERA.MX + "x" + CAMERA.MY)
			info.Add(FillSpaces(LOCALE.Get("INFO_GAMELANGUAGE") + " (F1/F2)") + ": " + LOCALE.language)
			info.Add(FillSpaces(LOCALE.Get("INFO_MEDIAMEMORYUSAGE")) + ": " + FormatINT(MEDIA.totalkb))

			info.Add(LOCALE.Get("INFO_KEYBOARD"), 10 * Scaler)
			info.Add("[1] " + LOCALE.Get("INFO_SETRESOLUTION") + " 800x600", 5 * Scaler)
			info.Add("[2] " + LOCALE.Get("INFO_SETRESOLUTION") + " 1024x768")
			info.Add("[3] " + LOCALE.Get("INFO_SETRESOLUTION") + " 1280x720")
			info.Add("[4] " + LOCALE.Get("INFO_SETRESOLUTION") + " 1920x1080")
			info.Add("[5] " + LOCALE.Get("INFO_SETRESOLUTION") + " " + SCREEN_DESKTOPWIDTH + "x" + SCREEN_DESKTOPHEIGHT + " (" + LOCALE.Get("INFO_FULLSCREEN") + ")")
			info.Add("[6] " + LOCALE.Get("INFO_SETRESOLUTION") + " " + SCREEN_DESKTOPWIDTH + "x" + SCREEN_DESKTOPHEIGHT + " (" + LOCALE.Get("INFO_FULLSCREENWINDOW") + ")")

			info.Add(LOCALE.Get("INFO_MOUSE"), 10 * Scaler)
			info.Add(FillSpaces(LOCALE.Get("INFO_MOUSEWHEEL")) + ": " + LOCALE.Get("INFO_LIGHTRANGE") + "=" + MouseWheel, 5 * Scaler)
			info.Add(FillSpaces(LOCALE.Get("INFO_MOUSELEFT")) + ": " + LOCALE.Get("INFO_SWITCHLIGHT") + " #1")
			info.Add(FillSpaces(LOCALE.Get("INFO_MOUSERIGHT")) + ": " + LOCALE.Get("INFO_SWITCHLIGHT") + " #2")

			info.Add(LOCALE.Get("INFO_FLAGS"), 10 * Scaler)
			info.Add(FillSpaces("[B] " + LOCALE.Get("INFO_BUMPMAPMODE")) + ": " + OnOff[TESTBUMP], 5 * Scaler)
			info.Add(FillSpaces("[C] " + LOCALE.Get("INFO_CAMERAMODE")) + ": " + CameraMode[FLAG.Get("CAMMODE")])
			info.Add(FillSpaces("[^] " + LOCALE.Get("FLAG_DELAY")) + ": " + OnOff[FLAG.Get("DELAY")])
			info.Add(FillSpaces("[V] " + LOCALE.Get("FLAG_VSYNC")) + ": " + OnOff[FLAG.Get("VSYNC")])
			info.Add(FillSpaces("[I] " + LOCALE.Get("INFO_SHOWGUI")) + ": " + OnOff[FLAG.Get("GUI")])			
			info.Add(FillSpaces("[F11] " + LOCALE.Get("INFO_SHOWCURSOR")) + ": " + OnOff[FLAG.Get("CURSOR")])
			info.Add(FillSpaces("[F12] " + LOCALE.Get("INFO_SHOWFPS")) + ": " + OnOff[FLAG.Get("FPS")])
			info.Add(FillSpaces("[TAB] " + LOCALE.Get("INFO_MEDIAQUALITY")) + ": " + QualityMode[FLAG.Get("QUALITY")])
			info.Add(FillSpaces("[NUM-] " + LOCALE.Get("INFO_SCREENSHOT")) + ": " + Spaces(GAME_SCREENSHOT_NUMBER, 4, "0"))
			info.Add(FillSpaces("[SPACE] " + LOCALE.Get("INFO_WIREFRAME")) + ": " + OnOff[FLAG.Get("WIREFRAME")])

			info.Update()

			' --------------------------------------------------------------------------------------------
			' Text Blip output
			' --------------------------------------------------------------------------------------------
			UpdateBlip()

			' --------------------------------------------------------------------------------------------
			' Red Canvas Frame to check if the ingame resolution switch works correct
			' --------------------------------------------------------------------------------------------
			If PROJECT_DEBUG Then

				' Begin drawing GL Lines
				glDisable(GL_TEXTURE_2D)
				glEnable(GL_COLOR_MATERIAL)
				glBegin(GL_LINES)
				glLineWidth(0.5)

				' draw a open rec rectangle
				SetColor(255, 0, 0)
				DrawGRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)

				' draw a crosshair
				SetAlpha(0.125)
				SetColor(255, 255, 255)
				DrawLine(SCREEN_WIDTH / 2, 0, SCREEN_WIDTH / 2, SCREEN_HEIGHT)
				DrawLine(0, SCREEN_HEIGHT / 2, SCREEN_WIDTH, SCREEN_HEIGHT / 2)

				' End drawing GL Lines
				glEnd()
				glDisable(GL_COLOR_MATERIAL)
				glEnable(GL_TEXTURE_2D)

			EndIf

		EndIf

		' --------------------------------------------------------------------------------------------
		' FPS output
		' --------------------------------------------------------------------------------------------
		If FLAG.Get("FPS") Then

			Font("FPS")
			SetColor(255, 255, 0)
			SetAlpha(1.0)
			SetBlend ALPHABLEND
			DrawText FPS.FPS, SCREEN_WIDTH - (TextWidth(FPS.FPS) * Scaler), 3

		EndIf


		' --------------------------------------------------------------------------------------------
		' Cursor Image
		' --------------------------------------------------------------------------------------------
		If FLAG.Get("CURSOR") Then

			SetBlend ALPHABLEND
			SetColor(255, 255, 255)
			SetScale(Scaler * 0.5, Scaler * 0.5)
			AutoMidHandle(False)
			DrawImage(MEDIA.GetImage("CURSORIMAGE"), CAMERA.MX, CAMERA.MY)

		EndIf
		
	End2D()

End Function



' ------------------------------------------------------------------------------------------------
' Initialize Blip Text
' ------------------------------------------------------------------------------------------------
Function InitBlip()

	TEST_TEXT = "OpenB3D Framework Demo~n(C) 2017 Krischan~nFeel free to contact me at www.blitzbasic.com~nGet the OpenB3D Wrapper at https://github.com/markcwm/openb3d.mod"

	TEST_BLIP = New TBeat.Create (40, DrawBlip)						' 40ms between charaters
	TEST_FADE = New TBeat.Create (6, FadeBlip)						' 6ms between fadeout steps
	TEST_DIGITS = 0													' counter for digits
	TEST_ALPHA = 1.0												' initial alpha value
	TEST_OUTPUT = ""												' set final output to nothing
	TEST_TIMER:Int = MilliSecs() + (Len(TEST_TEXT) * 120)			' time to live after last digit

End Function



' ------------------------------------------------------------------------------------------------
' Update Blip Text
' ------------------------------------------------------------------------------------------------
Function UpdateBlip()

	Local sx:Int = SCREEN_WIDTH / 32
	Local sy:Int = SCREEN_HEIGHT / 1.7

	'SetScale(Scaler, Scaler)

	If TSound(MEDIA.GetSound("BLIP")) Then TEST_CHANNEL = CueSound(MEDIA.GetSound("BLIP")) Else StopChannel TEST_CHANNEL

	Font("LARGE", TEST_ALPHA)

	' shadow
	SetColor 0, 0, 0
	DrawLines(TEST_OUTPUT, sx + (3 * Scaler), sy + (3 * Scaler))

	' color
	SetColor 0, 128, 255
	DrawLines(TEST_OUTPUT, sx, sy)

	'SetScale(Scaler, Scaler)
	SetAlpha(1.0)

End Function



' ------------------------------------------------------------------------------------------------
' Draw Blip Text with sound
' ------------------------------------------------------------------------------------------------
Function DrawBlip(Caller:TBeat, event:TEvent)

	If TEST_TEXT And MilliSecs() > initializing + 500 Then

		If TEST_DIGITS < Len(TEST_TEXT) Then

			If FLAG.Get("GUI") Then
			
				TEST_DIGITS:+1
				TEST_OUTPUT = Upper(Left(TEST_TEXT, TEST_DIGITS))

				Local c:Int = Asc(Right(TEST_OUTPUT, 1))

				SetChannelRate(TEST_CHANNEL, 2.0)

				If c <> 32 Then

					If TSound(MEDIA.GetSound("BLIP")) Then

						TEST_CHANNEL = CueSound(MEDIA.GetSound("BLIP"))
						TEST_CHANNEL.SetVolume(1.0 / 16 * Rnd(0.9, 1.1))
						TEST_CHANNEL.SetRate(2.0 * Rnd(0.995, 1.005))
						ResumeChannel TEST_CHANNEL

					Else

						StopChannel TEST_CHANNEL

					End If

				EndIf

			EndIf

		EndIf

	EndIf

End Function



' ------------------------------------------------------------------------------------------------
' Fade out Blip Text after a predefined period of time
' ------------------------------------------------------------------------------------------------
Function FadeBlip(Caller:TBeat, event:TEvent)

	If MilliSecs() > TEST_TIMER Then TEST_ALPHA:*0.99

End Function