' ------------------------------------------------------------------------------------------------
' Initialize Game
' ------------------------------------------------------------------------------------------------
Function InitGame()



	' --------------------------------------------------------------------------------------------
	' Application Title
	' --------------------------------------------------------------------------------------------
	AppTitle = PROJECT_TITLE + " " + PROJECT_VERSION



	' --------------------------------------------------------------------------------------------
	' Start logfile
	' --------------------------------------------------------------------------------------------
	LogInit()
	GetScreenshotNumber()


	' --------------------------------------------------------------------------------------------
	' Load Localization
	' --------------------------------------------------------------------------------------------
	LOCALE.path = PATH_Locale
	LOCALE.language = GAME_LANGUAGE
	LOCALE.Load		
	InitLocaleArrays



	' --------------------------------------------------------------------------------------------
	' Set FLAGS
	' --------------------------------------------------------------------------------------------
	FLAG.Add("VSYNC", 1)											' VSYNC Flag
	FLAG.Add("DELAY", 1)											' Delay Flag
	FLAG.Add("WIREFRAME", 0)										' Wireframe Flag
	FLAG.Add("GUI", 1)												' GUI Flag
	FLAG.Add("CURSOR", 1)											' Cursor Flag
	FLAG.Add("FPS", 1)												' FPS measurement
	FLAG.Add("SUSPEND", 0)											' Suspend check
	FLAG.Add("QUALITY", 1)											' Quality Flag
	FLAG.Add("BUMPMAP", 1)											' Bumpmap Flag
	FLAG.Add("CAMMODE", 0)											' Camera Mode
	FLAG.Add("LIGHTSWITCH1", 1)										' Ambient Light switch
	FLAG.Add("LIGHTSWITCH2", 1)										' Camera Light switch



	' ------------------------------------------------------------------------------------------------
	' MAXGUI Initialization
	' ------------------------------------------------------------------------------------------------
	TIME.Start()

	' Selector Panel height below the logo
	Local ph:Int = 150

	' Load Project Logo
	MAXGUI_LOGO = LoadImage("include/logo/logo.png")
	Local w:Int = MAXGUI_LOGO.width
	Local h:Int = MAXGUI_LOGO.Height

	' Create MAXGUI Window
	MAXGUI_WINDOW = CreateWindow(PROJECT_TITLE, SCREEN_CENTERX - (SCREEN_WIDTH / 2), SCREEN_CENTERY - (SCREEN_HEIGHT / 2), w, h + ph, Null, WINDOW_CLIENTCOORDS)

	' Create MAXGUI Canvas
	MAXGUI_CANVAS = CreateCanvas(0, 0, w, h, MAXGUI_WINDOW)
	SetGadgetLayout MAXGUI_CANVAS, 1, 1, 1, 1
	ActivateGadget(MAXGUI_CANVAS)
	SetGraphics CanvasGraphics(MAXGUI_CANVAS)
	Graphics3D w, h, 0, 2, 60, -1, True
	UpdateScaler()

	TIME.Stop(LOCALE.Get("LOG_CREATECANVAS"))



	' --------------------------------------------------------------------------------------------
	' Create Camera
	' --------------------------------------------------------------------------------------------

	' Main Camera
	CAMERA:TCam = New TCam
	CAMERA.parent = PIVOT_CAMERA
	CAMERA.Add()
	Camera.RangeMin = 0.1
	CAMERA.RangeMax = 1000
	CAMERA.speed = 0.5
	CAMERA.X = 0.0
	CAMERA.Y = 0.0
	CAMERA.Z = 0.0
	CAMERA.CLScolor = [0, 0, 0]
	CAMERA.TurnSpeed = 5.0
	CAMERA.RollSpeed = 5.0
	CAMERA.MX = SCREEN_WIDTH / 2
	CAMERA.MY = SCREEN_HEIGHT / 2
	CAMERA.UpdatePosition()
	CAMERA.Update()



	' --------------------------------------------------------------------------------------------
	' Add Resolution selector to MAXGUI Window
	' --------------------------------------------------------------------------------------------
	MAXGUI_Settings_Panel = CreatePanel(25, h + 25, w - 50, 100, MAXGUI_WINDOW, PANEL_GROUP, "Demo Graphics settings")
	MAXGUI_Graphics_Resolution = CreateComboBox(w / 2 - 150, 10, 300, 32, MAXGUI_Settings_Panel)
	MAXGUI_Button_Launch = CreateButton("Launch Demo", w / 2 - 128 - 8, 50, 128, 24, MAXGUI_Settings_Panel)
	MAXGUI_Button_Cancel = CreateButton("Cancel", w / 2 + 8, 50, 128, 24, MAXGUI_Settings_Panel)
	MAXGUI_GetResolutions()

	' Preselect the Launch button
	ActivateGadget(MAXGUI_Button_Launch)

	' will contain the new resolution
	Local sw:Int
	Local sh:Int

	' loop until the user selects a resolution or quits
	Repeat

		' Wait for an Event
		Select WaitEvent()

			' Window has been closed
			Case EVENT_WINDOWCLOSE, EVENT_APPTERMINATE
										MAXGUI_Appclose()

			Case EVENT_GADGETACTION
										Select EventSource()

											Case MAXGUI_Button_Cancel
																		MAXGUI_Appclose()

											Case MAXGUI_Button_Launch

																		' retrieve selected resolution
																		Local r1:String = MAXGUI_Resolutions[SelectedGadgetItem(MAXGUI_Graphics_Resolution)]
																		Local r2:Int = Instr(r1, "x", 1)
																		sw = Int(Mid(r1, 0, r2))
																		sh = Int(Mid(r1, r2 + 1, Len(r1)))

																		' rescale Canvas
																		SetGadgetShape(MAXGUI_CANVAS, 0, 0, w, h + ph)
																		ActivateGadget(MAXGUI_CANVAS)
																		SetResolution(w, h + ph)
																		SetScale(Scaler, Scaler)

																		' set flag to leave the loop
																		MAXGUI_LAUNCH = True

										End Select

			' Draw 2D Image on 3D Canvas
			Case EVENT_GADGETPAINT

										MAXGUI_UpdateCanvas(MAXGUI_CANVAS)

										MAXGUI_Begin2D()

											DrawImage(MAXGUI_LOGO, 0, 0)

										MAXGUI_End2D()

										Flip True


		EndSelect

		GCCollect()

	Until MAXGUI_LAUNCH = True

	TIME.TotalStart(LOCALE.Get("LOG_INITIALIZESTART"))



	' --------------------------------------------------------------------------------------------
	' Load MEDIA Files
	' --------------------------------------------------------------------------------------------
	LoadMedia()



	' --------------------------------------------------------------------------------------------
	' Set new Resolution
	' --------------------------------------------------------------------------------------------
	SetResolution(sw, sh)



	' --------------------------------------------------------------------------------------------
	' Create Test Objects
	' --------------------------------------------------------------------------------------------

	' level
	TESTLEVEL = CreateCube()
	ScaleEntity TESTLEVEL, 20, 10, 20
	PositionEntity TESTLEVEL, 0, 9, 0
	EntityShininess TESTLEVEL, 0.75
	FlipMesh TESTLEVEL

	' sphere
	TESTSPHERE = CreateCubeSphere(8)
	PositionEntity TESTSPHERE, 0, 0.25, 10
	ScaleEntity TESTSPHERE, 1, 1, 1
	EntityShininess TESTSPHERE, 0.25

	' camera spotlight
	TESTLIGHT = CreateLight(3, CAMERA.cam)
	LightColor TESTLIGHT, 255, 255 * 0.8, 255 * 0.6
	LightRange TESTLIGHT, 1000
	LightConeAngles TESTLIGHT, 0, 135

	' target for camera spotlight
	TESTTARGET = CreatePivot(CAMERA.cam)
	PositionEntity TESTTARGET, 0, 0, 100

	' blue light
	AMBLIGHT = CreateLight(2)
	PositionEntity AMBLIGHT, 0, 4, 10
	RotateEntity AMBLIGHT, 90, 0, 0
	LightColor AMBLIGHT, 0, 0, 128
	LightRange AMBLIGHT, 1000

	' assign shader
	Local shader:TShader = MEDIA.GetShader("TESTSHADER")
	ShaderTexture(shader, MEDIA.GetTexture("TEST"), "colorMap", 0)
	ShaderTexture(shader, MEDIA.GetTexture("TEST_NRM"), "normalMap", 1)
	ShaderTexture(shader, MEDIA.GetTexture("TEST_NRM"), "specularMap", 2)
	SetShaderProperties(shader)

	ShadeEntity TESTLEVEL, shader
	ShadeEntity TESTSPHERE, shader



	' --------------------------------------------------------------------------------------------
	' Initialize Blip Text
	' --------------------------------------------------------------------------------------------
	InitBlip()



	' --------------------------------------------------------------------------------------------
	' Mouse
	' --------------------------------------------------------------------------------------------
	CenterMouse()
	HideMouse()

	TIME.TotalStop(LOCALE.Get("LOG_INITIALIZESTOP"))

End Function



' ------------------------------------------------------------------------------------------------
' Initialize Locale Arrays (for multilingual display of switches like on/off and so on)
' ------------------------------------------------------------------------------------------------
Function InitLocaleArrays()

	LOCALE.CreateArray("ARRAY_ONOFF", OnOff)
	LOCALE.CreateArray("ARRAY_YESNO", YesNo)
	LOCALE.CreateArray("ARRAY_CAMERAMODE", CameraMode)
	LOCALE.CreateArray("ARRAY_MEDIAQUALITY", QualityMode)

End Function



' ------------------------------------------------------------------------------------------------
' Set Shader properties
' ------------------------------------------------------------------------------------------------
Function SetShaderProperties(shader:TShader)

	SetFloat4(shader, "tangent", 0.1, 0.1, 0.1, 0.1)
	SetFloat3(shader, "emission", 0.025, 0.025, 0.025)
	SetFloat(shader, "attspec", 0.01)
	UseInteger(shader, "active", TESTBUMP)

	'SetFloat(shader, "lightradius[0].Float", 0.1)
	'SetFloat(shader, "lightradius[1].Float", 0.2)
	'SetFloat(shader, "texturescale", 8)
	'SetFloat4(shader, "vambient", 0, 0.0, 0, 0)

End Function