' ------------------------------------------------------------------------------------------------
' Change resolution (instant)
' ------------------------------------------------------------------------------------------------
Function SetResolution(width:Int, Height:Int, centered:Int = True)

	TIME.Start

	CAMERA.Update

	Local x:Float = CAMERA.OX
	Local y:Float = CAMERA.OY
	Local z:Float = CAMERA.OZ
	Local pitch:Float = CAMERA.OPitch
	Local yaw:Float = CAMERA.OYaw
	Local roll:Float = CAMERA.ORoll

	If centered Then

		SetGadgetShape MAXGUI_WINDOW, SCREEN_CENTERX - (width / 2), SCREEN_CENTERY - (Height / 2), width, Height

	Else

		SetGadgetShape MAXGUI_WINDOW, 0, 0, width, Height

	EndIf

	SCREEN_WIDTH = width
	SCREEN_HEIGHT = Height
	SCREEN_DEPTH = SCREEN_DESKTOPDEPTH
	SCREEN_HERTZ = SCREEN_DESKTOPHERTZ
	SCREEN_MODE = 1

	' reset camera
	CAMERA.width = SCREEN_WIDTH
	CAMERA.Height = SCREEN_HEIGHT
	CAMERA.MX = SCREEN_WIDTH / 2
	CAMERA.MY = SCREEN_HEIGHT / 2
	CAMERA.x = x
	CAMERA.y = y
	CAMERA.z = z
	CAMERA.pitch = pitch
	CAMERA.yaw = yaw
	CAMERA.roll = roll
	CAMERA.Update()

	' repositon camera
	PositionEntity CAMERA.CAM, x, y, z
	RotateEntity CAMERA.CAM, pitch, yaw, roll

	' recalculate Scale
	UpdateScaler(SCREEN_WIDTH)

	TIME.Stop(LOCALE.Get("LOG_SETRESOLUTION") + " " + width + "x" + Height)

End Function



' --------------------------------------------------------------------------------
' Update Screen Scale
' --------------------------------------------------------------------------------
Function UpdateScaler(Value:Int = 2560)

	Scaler = SCREEN_WIDTH * 1.0 / Value
	SCREEN_ASPECT = SCREEN_WIDTH * 1.0 / SCREEN_HEIGHT
	
	Relative = SCREEN_WIDTH * 1.0 / 2560

End Function



' --------------------------------------------------------------------------------
' Fixed BeginMax2D() to make instant resolution switch work with Max2D
' --------------------------------------------------------------------------------
Function Begin2D()

	Local x:Int, y:Int, w:Int, h:Int
	GetViewport(x, y, w, h)

	glDisable(GL_LIGHTING)
	glDisable(GL_DEPTH_TEST)
	glDisable(GL_SCISSOR_TEST)
	glDisable(GL_FOG)
	glDisable(GL_CULL_FACE)

	glMatrixMode GL_TEXTURE
	glLoadIdentity

	glMatrixMode GL_PROJECTION
	glLoadIdentity
	glOrtho 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -1, 1

	glMatrixMode GL_MODELVIEW
	glLoadIdentity

	SetViewport x, y, w, h

	Local MaxTex:Int
	glGetIntegerv(GL_MAX_TEXTURE_UNITS, Varptr(MaxTex))

	For Local Layer:Int = 0 Until MaxTex

		glActiveTexture(GL_TEXTURE0+Layer)

		glDisable(GL_TEXTURE_CUBE_MAP)
		glDisable(GL_TEXTURE_GEN_S)
		glDisable(GL_TEXTURE_GEN_T)
		glDisable(GL_TEXTURE_GEN_R)

		glDisable(GL_TEXTURE_2D)

	Next

	glActiveTexture(GL_TEXTURE0)

	glViewport(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)
	glScissor(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)

	glEnable GL_BLEND
	glEnable(GL_TEXTURE_2D)

End Function



' --------------------------------------------------------------------------------
' Fixed EndMax2D() to make instant resolution switch work with Max2D
' --------------------------------------------------------------------------------
Function End2D()

	glDisable(GL_TEXTURE_CUBE_MAP)
	glDisable(GL_TEXTURE_GEN_S)
	glDisable(GL_TEXTURE_GEN_T)
	glDisable(GL_TEXTURE_GEN_R)

	glDisable(GL_TEXTURE_2D)
	glDisable GL_BLEND

	TGlobal.EnableStates()

	TGlobal.alpha_enable[0] = 0	' alpha blending was disabled by Max2d (GL_BLEND)
	TGlobal.blend_mode[0] = 1	' force alpha blending
	TGlobal.fx1[0] = 0			' full bright/surface normals was enabled by EnableStates (GL_NORMAL_ARRAY)
	TGlobal.fx2[0] = 1			' vertex colors was enabled by EnableStates (GL_COLOR_ARRAY)

	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR)
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE)

	glClearDepth(1.0)
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

	glAlphaFunc(GL_GEQUAL, 0.5)

	For Local cam:TCamera=EachIn TCamera.cam_list
		
		 ' active camera - was if cam.hide[0]=0
		 If cam = TGlobal.camera_in_use
		
		 	' fog with Max2d fix
			cam.UpdateFog()
			Exit
			
		EndIf
		
	Next

End Function