Type TCam

	Field cam:TCamera										' camera entity
	Field parent:TEntity									' camera parent

	Field X:Float = 0.0										' Position X
	Field Y:Float = 0.0										' Position Y
	Field Z:Float = 0.0										' Position Z

	Field OX:Float = 0.0									' Old Position X
	Field OY:Float = 0.0									' Old Position Y
	Field OZ:Float = 0.0									' Old Position Z

	Field OPitch:Float = 0.0								' Old Pitch
	Field OYaw:Float = 0.0									' Old Yaw
	Field ORoll:Float = 0.0									' Old Roll

	Field Pitch:Float = 0.0									' Pitch
	Field Yaw:Float = 0.0									' Yaw
	Field Roll:Float = 0.0									' Roll

	Field CLScolor:Int[] = [0, 0, 0]						' Background color
	Field FOGcolor:Int[] = [0, 0, 0]						' Fog color

	Field FieldofView:Float = 90.0							' Field of View
	Field FOGmode:Int = False								' Fog Mode
	Field Zoom:Float = 1.0									' Zoom factor

	Field RangeMin:Float = 1.0								' Frustum start
	Field RangeMax:Float = 10000.0							' Frustum end
	Field FOGrangeMin:Float = 1.0							' Fog Frustum start
	Field FOGrangeMax:Float = 10000.0						' Fog Frustum end

	Field ViewX:Int = 0										' Viewport X Position
	Field ViewY:Int = 0										' Viewport Y Position
	Field width:Int = SCREEN_WIDTH							' Viewport width
	Field Height:Int = SCREEN_HEIGHT						' Viewport height

	Field MX:Int											' Mouse X Position
	Field MY:Int											' Mouse Y Position

	Field PX:Int											' Projected X
	Field PY:Int											' Projected Y

	Const QuatToEuler:Double = 0.00001

	Field Speed:Float = 1.0									' Camera Movespeed
	Field RollSpeed:Float = 1.0								' Camera Rollspeed
	Field TurnSpeed:Float = 200.0							' Camera Turnspeed (Mousemove)

	Field Xspeed:Float										' Current X-Vector Speed
	Field Yspeed:Float										' Current Y-Vector Speed
	Field Zspeed:Float										' Current Z-Vector Speed

	Field Glob:Int = False									' Local/Global flag


	' --------------------------------------------------------------------------------------------
	' Add new camera
	' --------------------------------------------------------------------------------------------
	Method Add()

		cam = CreateCamera(parent)

	End Method


	' --------------------------------------------------------------------------------------------
	' Show Camera
	' --------------------------------------------------------------------------------------------
	Method Show()

		ShowEntity cam

	End Method


	' --------------------------------------------------------------------------------------------
	' Hide Camera
	' --------------------------------------------------------------------------------------------
	Method Hide()

		HideEntity cam

	End Method


	' --------------------------------------------------------------------------------------------
	' Move Camera
	' --------------------------------------------------------------------------------------------
	Method Move(quaternion:Int = True)

		Local speed:Float = CAMERA.Speed * FPS.Multi
		'Local xspeed:Float, yspeed:Float, zspeed:Float

		If quaternion Then

			Local rolling:Float = -PlayerXSpeed * RollSpeed
			Local tx:Float = LimitFloat(Normalize(MX, 0, SCREEN_WIDTH - 0.5, 1, -1), -1, 1)
			Local ty:Float = LimitFloat(Normalize(MY, 0, SCREEN_HEIGHT - 0.5, -1, 1), -1, 1)

			tx = Sin(tx * 90) * 4
			ty = Sin(ty * 90) * 4

			If tx > - 0.01 And tx < 0.01 Then tx = 0
			If ty > - 0.01 And ty < 0.01 Then ty = 0

			xspeed = ty * TurnSpeed * FPS.multi
			yspeed = tx * TurnSpeed * FPS.multi
			zspeed = rolling * FPS.multi

			MoveEntity cam, 0, PLAYERYSPEED * Camera.Speed * FPS.multi, PlayerZSpeed * CAMERA.speed * FPS.multi

			Local Quat:TQuaternion = EulerToQuat(0.0, 0.0, 0.0)
			Local Turn_Quat:TQuaternion = EulerToQuat(0.0, 0.0, 0.0)

			Quat = EulerToQuat(EntityPitch(cam, True), EntityYaw(cam, True), EntityRoll(cam, True))
			Turn_Quat = EulerToQuat(Xspeed, Yspeed, zspeed)
			Quat = MultiplyQuats(Quat, Turn_Quat)
			Quat = NormalizeQuat(Quat)
			QuatToEuler2(Quat.x, Quat.y, Quat.z, Quat.w, Pitch, Yaw, Roll)

			RotateEntity cam, pitch, yaw, roll

		Else

			xspeed = ((MX - SCREEN_WIDTH / 2) - xspeed) / 2 + xspeed
			yspeed = ((MY - SCREEN_HEIGHT / 2) - yspeed) / 2 + yspeed

			CenterMouse()
			TurnEntity parent, 0, -xspeed * TurnSpeed / 25.0, 0
			TurnEntity cam, yspeed * TurnSpeed / 25.0, 0, 0
			MoveEntity parent, PlayerXSpeed * speed, PlayerYSpeed * speed, PlayerZSpeed * speed

		EndIf

	End Method


	' --------------------------------------------------------------------------------------------
	' Update Camera
	' --------------------------------------------------------------------------------------------
	Method Update()

		OX = EntityX(cam)
		OY = EntityY(cam)
		OZ = EntityZ(cam)

		OPitch = EntityPitch(cam)
		OYaw = EntityYaw(cam)
		ORoll = EntityRoll(cam)

		CameraRange cam, rangemin, rangemax
		CameraFogRange cam, FOGrangemin, FOGrangemax
		CameraClsColor cam, CLScolor[0], CLScolor[1], CLScolor[2]
		CameraZoom cam, zoom
		CameraViewport cam, viewx, viewy, width, Height

	End Method


	' --------------------------------------------------------------------------------------------
	' Update Camera Position
	' --------------------------------------------------------------------------------------------
	Method UpdatePosition()

		PositionEntity cam, x, y, z

	End Method


	' --------------------------------------------------------------------------------------------
	' Returns true if the Camera has been Moved since last position check
	' --------------------------------------------------------------------------------------------
	Method Moved:Int()

		If (ox <> x) Or (oy <> y) Or (oz <> z) Then Return True

		Return False

	End Method


	' --------------------------------------------------------------------------------------------
	' Calculate Screen Projection coordinates
	' --------------------------------------------------------------------------------------------
	Method Project(x:Float = 0.0, y:Float = 0.0, z:Float = 0.0)

		CameraProject cam, x, y, z
		PX = ProjectedX()
		PY = ProjectedY()

	End Method


	' --------------------------------------------------------------------------------------------
	' Store current camera position
	' --------------------------------------------------------------------------------------------
	Method GetCameraPosition(glob:Int = False)

		X = EntityX(cam, glob)
		Y = EntityY(cam, glob)
		Z = EntityZ(cam, glob)

	End Method

	' ----------------------------------------------------------------------------
	' help function of turncam
	' ----------------------------------------------------------------------------
	Function EulerToQuat:TQuaternion(pitch:Float, yaw:Float, roll:Float)

		Local cr:Float = Cos(-roll / 2.0)
		Local cp:Float = Cos(pitch / 2.0)
		Local cy:Float = Cos(yaw / 2.0)
		Local sr:Float = Sin(-roll / 2.0)
		Local sp:Float = Sin(pitch / 2.0)
		Local sy:Float = Sin(yaw / 2.0)
		Local cpcy:Float = cp * cy
		Local spsy:Float = sp * sy
		Local spcy:Float = sp * cy
		Local cpsy:Float = cp * sy

		Local q:TQuaternion = New TQuaternion

		q.w:Float = cr * cpcy + sr * spsy
		q.x:Float = sr * cpcy - cr * spsy
		q.y:Float = cr * spcy + sr * cpsy
		q.z:Float = cr * cpsy - sr * spcy

		Return q

	End Function


	' ----------------------------------------------------------------------------
	' help Function of turncam
	' ----------------------------------------------------------------------------
	Function QuatToEuler2(x:Float, y:Float, z:Float, w:Float, pitch:Float Var, yaw:Float Var, roll:Float Var)

		Local sint:Float = (2.0 * w * y) - (2.0 * x * z)
		Local cost_temp:Float = 1.0 - (sint * sint)
		Local cost:Float

		If Abs(cost_temp) > QuatToEuler

			cost = Sqr(cost_temp)

		Else

			cost = 0.0

		EndIf

		Local sinv:Float, cosv:Float, sinf:Float, cosf:Float

		If Abs(cost) > QuatToEuler

			sinv = ((2.0 * y * z) + (2.0 * w * x)) / cost
			cosv = (1.0 - (2.0 * x * x) - (2.0 * y * y)) / cost
			sinf = ((2.0 * x * y) + (2.0 * w * z)) / cost
			cosf = (1.0 - (2.0 * y * y) - (2.0 * z * z)) / cost

		Else

			sinv = (2.0 * w * x) - (2.0 * y * z)
			cosv = 1.0 - (2.0 * x * x) - (2.0 * z * z)
			sinf = 0.0
			cosf = 1.0

		EndIf

		pitch = ATan2(sint, cost)
		yaw = ATan2(sinf, cosf)
		roll = -ATan2(sinv, cosv)

	End Function
	
	
	' ----------------------------------------------------------------------------
	' help Function of Move Method
	' ----------------------------------------------------------------------------
	Function MultiplyQuats:TQuaternion(q1:TQuaternion, q2:TQuaternion)

		Local q:TQuaternion = New TQuaternion

		q.w = q1.w * q2.w - q1.x * q2.x - q1.y * q2.y - q1.z * q2.z
		q.x = q1.w * q2.x + q1.x * q2.w + q1.y * q2.z - q1.z * q2.y
		q.y = q1.w * q2.y + q1.y * q2.w + q1.z * q2.x - q1.x * q2.z
		q.z = q1.w * q2.z + q1.z * q2.w + q1.x * q2.y - q1.y * q2.x

		Return q

	End Function


	' ----------------------------------------------------------------------------
	' help Function of Move Method
	' ----------------------------------------------------------------------------
	Function NormalizeQuat:TQuaternion(q:TQuaternion)

		Local uv:Float = Sqr(q.w * q.w + q.x * q.x + q.y * q.y + q.z * q.z)

		q.w = q.w / uv
		q.x = q.x / uv
		q.y = q.y / uv
		q.z = q.z / uv

		Return q

	End Function

End Type