Type TMedia

	Global media:TMap = CreateMap()

	Field desc:String
	Field cat:String
	Field obj:Object
	Field option:Int
	Field path:String
	Field kb:Long
	Field err:Int
	Field vert:String
	Field frag:String

	Field LoaderAnimation:TImage	
	Field LoaderProgress:Float

	Field ShowAnimation:Int = True

	Global totalkb:Int



	' ------------------------------------------------------------------------------------------------
	' Update Media Loader
	' ------------------------------------------------------------------------------------------------
	Method Update(file:String = Null)

		' Update Animation
		LoaderProgress:+Rnd(0.25, 2.5)

		Cls

		RenderWorld

		Begin2D()

			SetColor(0, 255, 0)
			SetBlend ALPHABLEND

			'SetScale(Scaler, Scaler)
			Font("LARGE")

			If LoaderAnimation And ShowAnimation Then

				Local w:Int = LoaderAnimation.width
				Local h:Int = LoaderAnimation.Height

				SetRotation(loaderprogress)
				SetColor(((Sin(LOADERPROGRESS) * Cos(LOADERPROGRESS)) + 1) / 2.0 * 255, (Cos(LOADERPROGRESS) + 1) / 2.0 * 255, (Sin(LOADERPROGRESS) + 1) / 2.0 * 255)

				MidHandleImage(LoaderAnimation)
				DrawImage(LoaderAnimation, SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2)

			EndIf

			SetRotation(0)
			SetColor(0, 255, 0)

			'Delay 1000

			DrawText file, SCREEN_WIDTH / 2 - TextWidth(file) / 2, SCREEN_HEIGHT / 2 - TextHeight(file) / 2

		End2D()

		Flip False

		If KeyHit(KEY_ESCAPE) Then End

	End Method



	' ------------------------------------------------------------------------------------------------
	' Create a Media Object
	' ------------------------------------------------------------------------------------------------
	Method Create:TMedia(cat:String, desc:String, w:Int = 0, h:Int = 0, option:Int = Null)

		Local m:TMedia = New Tmedia

		m.desc = desc
		m.cat = cat
		m.path = "NEW"

		Select cat

			Case "PIX" m.obj = CreatePixmap(w, h, option)
			Case "TEX" m.obj = CreateTexture(w, h, option)
			Case "SPR" m.obj = CreateSprite()
			Case "MOD" m.obj = CreateMesh()

		End Select

		If m.obj Then

			m.kb = (w * h * 4)
			totalkb:+m.kb
			LogUpdate(FormatText(TIME.ElapsedTime()) + ": " + LOCALE.Get("LOG_MEDIACREATED") + ": " + m.desc)
			MapInsert(MEDIA, desc, m)

			Update(desc + " " + m.path)

		EndIf

		Return m

	End Method


	' ------------------------------------------------------------------------------------------------
	' Create a Media Object
	' ------------------------------------------------------------------------------------------------
	Method PixToTex:TMedia(cat:String, desc:String, pixmap:TPixmap)

		Local m:TMedia = New Tmedia

		m.desc = desc
		m.cat = cat
		m.path = "NEW"

		Local w:Int = pixmap.width
		Local h:Int = pixmap.Height

		m.obj = CreateTexture(w, h)		
		BufferToTex TTexture(m.obj), PixmapPixelPtr(pixmap, 0, 0)

		If m.obj Then

			m.kb = (w * h * 4)
			totalkb:+m.kb
			LogUpdate(FormatText(TIME.ElapsedTime()) + ": " + LOCALE.Get("LOG_MEDIACREATED") + ": " + m.desc)
			MapInsert(MEDIA, desc, m)
		
			Update(desc + " " + m.path)

		EndIf

		Return m

	End Method


	' ------------------------------------------------------------------------------------------------
	' Assign a Mesh Entity to a Media Object
	' ------------------------------------------------------------------------------------------------
	Method AssignMesh:TMedia(e:TMesh, desc:String)

		Local m:TMedia = New Tmedia
		m.desc = desc
		m.cat = "M"
		m.obj = e

		LogUpdate(LOCALE.Get("LOG_MESHASSIGNED") + ": " + m.desc)

		MapInsert(MEDIA, desc, m)

		Return m

	End Method


	' ------------------------------------------------------------------------------------------------
	' Assign a Light Entity to a Media Object
	' ------------------------------------------------------------------------------------------------
	Method AssignLight:TMedia(e:TLight, desc:String)

		Local m:TMedia = New Tmedia
		m.desc = desc
		m.cat = "L"
		m.obj = e

		LogUpdate(LOCALE.Get("LOG_LIGHTASSIGNED") + ": " + m.desc)

		MapInsert(MEDIA, desc, m)

		Return m

	End Method


	' ------------------------------------------------------------------------------------------------
	' Load a Media Object
	' ------------------------------------------------------------------------------------------------
	Method Load:TMedia(cat:String, desc:String, file:String, option:Int = Null, showprogess:Int = True, framewidth:Int = 0, frameheight:Int = 0, firstframe:Int = 1, framecount:Int = 1)

		Local m:TMedia = New Tmedia
		Local vert:String
		Local frag:String

		m.desc = desc
		m.cat = cat
		m.option = option
		m.kb = 0
		m.err = False

		Local check:TStream = OpenStream(m.path)

		If Not check Then m.err = True Else CloseStream check 

		If Not err Then

			Select cat

				Case "FNT"
					m.path = PATH_Font + file
					m.obj = LoadImageFont(m.path, option, SMOOTHFONT)

				Case "PIX"
					m.path = PATH_Texture + file
					m.obj = LoadPixmap(m.path)

				Case "IMG"
					m.path = PATH_Images + file
					m.obj = LoadImage(m.path)

				Case "SND"
					m.path = PATH_Sound + file
					m.obj = LoadSound(m.path, option)

				Case "TEX"
					m.path = PATH_Texture + file
					m.obj = LoadTexture(m.path, option)

				Case "ANI"
					m.path = PATH_Texture + file
					m.obj = LoadAnimTexture(PATH_Texture + file, option, framewidth, frameheight, firstframe, framecount)

				Case "MOD"
					m.path = PATH_Model + file
					m.obj = LoadAnimMesh(m.path)

				Case "SPR"
					m.path = PATH_Texture + file
					m.obj = LoadSprite(m.path, option)

				Case "SHA"
					m.path = PATH_SHADER + file

					vert = m.path + ".vert.glsl"
					frag = m.path + ".frag.glsl"

					m.vert = ReadString(ReadFile(vert), FileSize(vert))
					m.frag = ReadString(ReadFile(frag), FileSize(frag))

					m.obj = CreateShader("", m.vert, m.frag)

			End Select

			If m.obj Then

				If cat = "SHA" Then

					m.kb = FileSize(vert) + FileSize(frag)

				Else

					m.kb = FileSize(m.path)

				End If

				totalkb:+m.kb
				LogUpdate(FormatText(TIME.ElapsedTime()) + ": " + cat + " [" + desc + "] = " + m.path)

			Else

				Select m.cat

					Case "TEX" m.obj = GetTexture("NOTEX")
					Case "ANI" m.obj = GetTexture("NOTEX")

				End Select

				LogUpdate(LOCALE.Get("LOG_ERRORLOADING") + ": " + m.path)

			End If

			If showprogess Then Update(desc + " " + m.path)

			MapInsert(MEDIA, desc, m)

			Return m

		EndIf

	End Method


	' ------------------------------------------------------------------------------------------------
	' Unloads a MEDIA Object And (hopyfully) frees the memory it used
	' ------------------------------------------------------------------------------------------------
	Method Unload(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		Select m.cat

			Case "ANI", "TEX"
								If TTexture(m.obj) Then

									' OpenB3D Hack to really free the VRAM!
									glDeleteTextures 1, TTexture(m.obj).texture
									FreeTexture TTexture(m.obj)
									ReleaseObject(m)

								EndIf
			Case "SND"
								If TSound(m.obj) Then

									ReleaseObject(m)

								EndIf
			Case "MOD"
								If TMesh(m.obj) Then

									FreeEntity TMesh(m.obj)
									ReleaseObject(m)

								EndIf
			Case "FNT"
								If TImageFont(m.obj) Then

									ReleaseObject(m)

								EndIf

		End Select	

	End Method



	' ------------------------------------------------------------------------------------------------
	' Deletes a TMedia Object, used by the Unload Method
	' ------------------------------------------------------------------------------------------------
	Method ReleaseObject(m:TMedia)

		totalkb:-m.kb
		m.obj = Null
		m = Null

	End Method



	' ------------------------------------------------------------------------------------------------
	' Returns Texture Object
	' ------------------------------------------------------------------------------------------------
	Method GetTexture:TTexture(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If m.cat = "TEX" Or m.cat = "ANI" Then Return TTexture(m.obj)

		Else

			Return TTexture(GetTexture("NOTEX"))

		EndIf

		Return Null

	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Pixmap Object
	' ------------------------------------------------------------------------------------------------
	Method GetPixmap:TPixmap(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m Then

			If m.cat = "PIX" Then Return TPixmap(m.obj)

		EndIf

		Return Null

	End Method



	' ------------------------------------------------------------------------------------------------
	' Returns Image Object
	' ------------------------------------------------------------------------------------------------
	Method GetImage:TImage(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m Then

			If m.cat = "IMG" Then Return TImage(m.obj)

		EndIf

		Return Null

	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Sound Object
	' ------------------------------------------------------------------------------------------------
	Method GetSound:TSound(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m Then

				If m.cat = "SND" Then Return TSound(m.obj)

		EndIf

		Return Null

	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Font Object
	' ------------------------------------------------------------------------------------------------
	Method GetFont:TImageFont(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If m.cat = "FNT" Then Return TImageFont(m.obj)

		EndIf

		Return Null

	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Mesh Object
	' ------------------------------------------------------------------------------------------------
	Method GetMesh:TMesh(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If m.cat = "MOD" Then Return TMesh(m.obj)

		EndIf

		Return Null

	End Method

	' ------------------------------------------------------------------------------------------------
	' Returns Sprite Object
	' ------------------------------------------------------------------------------------------------
	Method GetSprite:TSprite(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If m.cat = "SPR" Then Return TSprite(m.obj)

		EndIf

		Return Null

	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Light Object
	' ------------------------------------------------------------------------------------------------
	Method GetLight:TLight(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If m.cat = "LIG" Then Return TLight(m.obj)

		EndIf

		Return Null

	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Light Object
	' ------------------------------------------------------------------------------------------------
	Method GetShader:TShader(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If m.cat = "SHA" Then Return TShader(m.obj)

		EndIf

		Return Null

	End Method

	' ------------------------------------------------------------------------------------------------
	' Returns Options of an Object
	' ------------------------------------------------------------------------------------------------
	Method GetOption:Int(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))

		If m Then

			If Key = m.desc Then Return m.option

		EndIf

		Return Null

	End Method

End Type