' ------------------------------------------------------------------------------------------------
' Project
' ------------------------------------------------------------------------------------------------
Const PROJECT_TITLE:String = "OpenB3D Framework Demo"			' Project: Title
Const PROJECT_VERSION:Float = 1.000								' Project: Version
Const PROJECT_DEBUG:Int = False									' Project: Debug Mode

' ------------------------------------------------------------------------------------------------
' Additional Key Codes
' ------------------------------------------------------------------------------------------------
Const KEY_CFLEX:Int = 220										' Missing Circumflex Key code


' ------------------------------------------------------------------------------------------------
' Texture QUality
' ------------------------------------------------------------------------------------------------
Global GAME_QUALITY:String = "HQ"								' Texture Quality Flag


' ------------------------------------------------------------------------------------------------
' Paths
' ------------------------------------------------------------------------------------------------
Global PATH_FONT:String = "fonts/"								' Path: Font
Global PATH_Images:String = "gfx/"								' Path: Images (all non-Textures)
Global PATH_MODEL:String = "models/" + GAME_QUALITY + "/"		' Path: Models
Global PATH_TEXTURE:String = "textures/" + GAME_QUALITY + "/"	' Path: Texture
Global PATH_SOUND:String = "sounds/" + GAME_QUALITY + "/"		' Path: Sound
Global PATH_SHADER:String = "shader/"							' Path: Sound
Global PATH_SCREENSHOT:String = "screenshots/"					' Path: Screenshots
Global PATH_LOCALE:String = "locale/"							' Path: Localization Database


' ------------------------------------------------------------------------------------------------
' Game Global Values
' ------------------------------------------------------------------------------------------------
Global GAME_SEED:Int = 42										' Game: Initial Seed
Global GAME_SPEED:Float = 1.0/100								' Game: Speed Multiplicator
Global GAME_LANGUAGE:String = "EN"								' Game: Language for Localization
Global GAME_SCREENSHOT_NUMBER:Int = 0							' Game: Screenshot Number start


' ------------------------------------------------------------------------------------------------
' Screen Globals
' ------------------------------------------------------------------------------------------------
Global SCREEN_WIDTH:Int = 800									' Screen Width
Global SCREEN_HEIGHT:Int = 400	 								' Screen Height
Global SCREEN_DEPTH:Int = 32									' Screen Depth
Global SCREEN_HERTZ:Int = 60									' Screen Refresh Rate
Global SCREEN_MODE:Int = 2										' Screen Mode
Global SCREEN_DESKTOPWIDTH:Int = DesktopWidth()					' Desktop Center horizontal
Global SCREEN_DESKTOPHEIGHT:Int = DesktopHeight()				' Desktop Center vertical
Global SCREEN_DESKTOPDEPTH:Int = DesktopDepth()					' Desktop Bit Depth
Global SCREEN_DESKTOPHERTZ:Int = DesktopHertz()					' Desktop Hertz rate
Global SCREEN_CENTERX:Int = SCREEN_DESKTOPWIDTH / 2				' Desktop Center horizontal
Global SCREEN_CENTERY:Int = SCREEN_DESKTOPHEIGHT / 2			' Desktop Center vertical
Global SCREEN_TIMER:TTimer = CreateTimer(1000)					' Maximum Refresh Rate
Global SCREEN_ASPECT:Float = SCREEN_WIDTH * 1.0 / SCREEN_HEIGHT	' Screen Aspect Ratio
Global Scaler:Float												' Global Screen Scale Factor
Global Relative:Float											' Global Screen Scale Factor (rel.)


' ------------------------------------------------------------------------------------------------
' Localization
' ------------------------------------------------------------------------------------------------
Global LOCALE:TLocale = New TLocale								' Language Localization Database


' ------------------------------------------------------------------------------------------------
' Logging
' ------------------------------------------------------------------------------------------------
Global INI_LogToFile:Int = True									' Logging to File Flag
Global INI_LogToConsole:Int = True								' Logging to Console Flag


' ------------------------------------------------------------------------------------------------
' MaxGUI
' ------------------------------------------------------------------------------------------------
Global MAXGUI_WINDOW:TGadget									' MAXGUI: Main Window
Global MAXGUI_CANVAS:TGadget									' MaxGUI: Canvas
Global MAXGUI_Settings_Panel:TGadget							' MAXGUI: Settings Panel
Global MAXGUI_Graphics_Resolution:TGadget						' MAXGUI: Resolution Pulldown
Global MAXGUI_Button_Launch:TGadget								' MAXGUI: Launch Button
Global MAXGUI_Button_Cancel:TGadget								' MAXGUI: Cancel Button
Global MAXGUI_LAUNCH:Int										' MAXGUI: Launch Flag
Global MAXGUI_LOGO:TImage										' MAXGUI: Logo
Global MAXGUI_Timer:TTimer = CreateTimer(60)					' MAXGUI: Timer

' ------------------------------------------------------------------------------------------------
' Types
' ------------------------------------------------------------------------------------------------
Global FLAG:TFlag = New TFlag									' Flag Management
Global FPS:TFPS = New TFPS										' FPS Counter
Global info:TInfo = New TInfo									' Information Text
Global MEDIA:TMedia = New TMedia								' Media Management
Global TIME:TTime = New TTime									' Time measurement


' ------------------------------------------------------------------------------------------------
' Timed VSYNC
' ------------------------------------------------------------------------------------------------
Global TIMER_Safe:Int = SCREEN_HERTZ							' Safe FPS limit
Global TIMER_Update:Int = 0										' Update Frequency
Global TIMER_Interval:Int = 0									' Update Interval Helper
Global TIMER_Timer:Int = MilliSecs()							' Current Timer
Global TIMER_Lasttimer:Int = MilliSecs()						' Last Timer


' ------------------------------------------------------------------------------------------------
' TLocale Array Switches
' ------------------------------------------------------------------------------------------------
Global OnOff:String[]											' simple on/off Array (from Locale)
Global YesNo:String[]											' simple yes/no Array (from Locale)
Global CameraMode:String[]  									' Camera Mode (from Locale)
Global QualityMode:String[]										' Quality Mode (from Locale)


' ------------------------------------------------------------------------------------------------
' Input
' ------------------------------------------------------------------------------------------------
Global MouseWheel:Int = 1										' Position of Mousewheel
Global MouseButton:Int											' Mousebutton State


' ------------------------------------------------------------------------------------------------
' Moving
' ------------------------------------------------------------------------------------------------
Global PlayerXSpeed:Int											' Player X Movement Activity
Global PlayerYSpeed:Int											' Player Y Movement Activity
Global PlayerZSpeed:Int											' Player Z Movement Activity


' ------------------------------------------------------------------------------------------------
' Cameras
' ------------------------------------------------------------------------------------------------
Global CAMERA:TCam												' Main Camera (Camera.cam)
Global PIVOT_CAMERA:TPivot = CreatePivot()						' Pivot for Main Camera


' ------------------------------------------------------------------------------------------------
' Sound Channels
' ------------------------------------------------------------------------------------------------
Global TEST_CHANNEL:TChannel = New TChannel						' Test Channel


' ------------------------------------------------------------------------------------------------
' Test Objects
' ------------------------------------------------------------------------------------------------
Global TESTTARGET:TEntity										' Target for Camera Spotlight
Global TESTLEVEL:TMesh											' Testlevel
Global TESTSPHERE:TMesh											' Sphere in Testlevel
Global TESTLIGHT:TLight											' Camera Spotlight
Global AMBLIGHT:TLight											' Ambient Light above the Sphere


' ------------------------------------------------------------------------------------------------
' Other
' ------------------------------------------------------------------------------------------------
Global TESTBUMP:Int												' Flag for Bumpmapping
Global TESTCOUNTER:Float										' Counter

Global TEST_BLIP:TBeat
Global TEST_FADE:TBeat
Global TEST_TIMER:Int
Global TEST_DIGITS:Int
Global TEST_TEXT:String
Global TEST_OUTPUT:String
Global TEST_ALPHA:Float